import sinon from 'sinon';
import { service as agreements } from './agreements.js';

let urlParams;
let spy;

describe('Billing Agreements Module', () => {
  beforeEach(() => {
    spy = sinon.spy();
    urlParams = { affiliationCode: '321321321' };
  });

  afterEach(() => {
    spy = undefined;
    urlParams = undefined;
  });

  it('Should have methods after initialization.', () => {
    const service = agreements();
    expect(service.get).to.be.a('function');
  });

  it('Should call get request once with correct params.', () => {
    const getRequest = () => spy;
    const service = agreements(getRequest);
    service.get(urlParams);

    expect(spy).to.have.been.calledOnceWith(urlParams);
  });

  it('Should return a valid URL with affiliationCode.', () => {
    urlParams = { affiliationCode: '1234' };

    const getRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = agreements(getRequest);
    service.get(urlParams);

    expect(spy).to.have.been.calledWith(
      'v0/merchants/1234/billing/agreements/?pageSize=30');
  });

  it('Should return a valid URL with affiliationCode and agreementId.', () => {
    urlParams = {
      affiliationCode: '1234',
      agreementId: '9876',
    };

    const getRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = agreements(getRequest);
    service.get(urlParams);

    expect(spy).to.have.been.calledWith(
      'v0/merchants/1234/billing/agreements/9876?pageSize=30');
  });

  it('Should return a valid URL with document.', () => {
    urlParams = { document: '9999' };

    const getRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = agreements(getRequest);
    service.get(urlParams);

    expect(spy).to.have.been.calledWith(
      'v0/merchants/billing/9999/agreements/?pageSize=30');
  });

  it('Should return a valid URL with document and agreementId.', () => {
    urlParams = {
      document: '9999',
      agreementId: '9876',
    };

    const getRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = agreements(getRequest);
    service.get(urlParams);

    expect(spy).to.have.been.calledWith(
      'v0/merchants/billing/9999/agreements/9876?pageSize=30');
  });

  it('Should ignore document if affiliationCode is set.', () => {
    urlParams = {
      affiliationCode: '1234',
      document: '9999',
      agreementId: '9876',
    };

    const getRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = agreements(getRequest);
    service.get(urlParams);

    expect(spy).to.have.been.calledWith(
      'v0/merchants/1234/billing/agreements/9876?pageSize=30');
  });

  it('Should accept a custom pageSize.', () => {
    urlParams = {
      document: '9999',
      pageSize: 10,
    };

    const getRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = agreements(getRequest);
    service.get(urlParams);

    expect(spy).to.have.been.calledWith(
      'v0/merchants/billing/9999/agreements/?pageSize=10');
  });

  it('Should accept extra parameters.', () => {
    urlParams = {
      document: '9999',
      startDate: '2020-03-12',
    };

    const getRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = agreements(getRequest);
    service.get(urlParams);

    expect(spy).to.have.been.calledWith(
      'v0/merchants/billing/9999/agreements/?pageSize=30&startDate=2020-03-12');
  });
});
